use std::{
    fs::File,
    io::{Read, Result},
    ops::Deref,
};

fn load_data() -> Result<String> {
    let mut file = File::open("./data").expect("Data file not found");
    let mut content = String::new();
    file.read_to_string(&mut content)?;
    Ok(content)
}

fn count_chars_to_firt_sop_marker(data: &str) -> usize {
    let arr: Vec<_> = data.chars().collect();
    for i in 3..arr.len() {
        let mut char: [char; 4] = ['a'; 4];
        char.clone_from_slice(&arr[i - 3..=i]);
        let mut char: Vec<char> = char.into();
        char.sort();
        char.dedup();
        println!("{:?}", char);
        if char.len() == 4 {
            return i + 1;
        }
    }
    10
}

fn count_chars_to_firt_som_marker(data: &str) -> usize {
    let arr: Vec<_> = data.chars().collect();
    for i in 13..arr.len() {
        let mut char: [char; 14] = ['a'; 14];
        char.clone_from_slice(&arr[i - 13..=i]);
        let mut char: Vec<char> = char.into();
        char.sort();
        char.dedup();
        println!("{:?}", char);
        if char.len() == 14 {
            return i + 1;
        }
    }
    10
}

fn main() {
    let data = load_data().unwrap();
    let packet_point = count_chars_to_firt_sop_marker(&data);
    let message_point = count_chars_to_firt_som_marker(&data);
    println!("start of packet: {}", packet_point);
    println!("start of message: {}", message_point)
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn sop1() {
        let data = String::from("bvwbjplbgvbhsrlpgdmjqwftvncz");
        let point = count_chars_to_firt_sop_marker(&data);
        assert_eq!(point, 5);
    }
    #[test]
    fn sop2() {
        let data = String::from("nppdvjthqldpwncqszvftbrmjlhg");
        let point = count_chars_to_firt_sop_marker(&data);
        assert_eq!(point, 6);
    }
    #[test]
    fn sop3() {
        let data = String::from("nznrnfrfntjfmvfwmzdfjlvtqnbhcprsg");
        let point = count_chars_to_firt_sop_marker(&data);
        assert_eq!(point, 10);
    }
    #[test]
    fn sop4() {
        let data = String::from("zcfzfwzzqfrljwzlrfnpqdbhtmscgvjw");
        let point = count_chars_to_firt_sop_marker(&data);
        assert_eq!(point, 11);
    }

    #[test]
    fn som1() {
        let data = String::from("mjqjpqmgbljsphdztnvjfqwrcgsmlb");
        let point = count_chars_to_firt_som_marker(&data);
        assert_eq!(point, 19);
    }
    #[test]
    fn som2() {
        let data = String::from("bvwbjplbgvbhsrlpgdmjqwftvncz");
        let point = count_chars_to_firt_som_marker(&data);
        assert_eq!(point, 23);
    }
    #[test]
    fn som3() {
        let data = String::from("nppdvjthqldpwncqszvftbrmjlhg");
        let point = count_chars_to_firt_som_marker(&data);
        assert_eq!(point, 23);
    }
    #[test]
    fn som4() {
        let data = String::from("nznrnfrfntjfmvfwmzdfjlvtqnbhcprsg");
        let point = count_chars_to_firt_som_marker(&data);
        assert_eq!(point, 29);
    }
    #[test]
    fn som5() {
        let data = String::from("zcfzfwzzqfrljwzlrfnpqdbhtmscgvjw");
        let point = count_chars_to_firt_som_marker(&data);
        assert_eq!(point, 26);
    }
}
